# R client to the gypsum REST API

|Environment|Status|
|---|---|
|[BioC-release](https://bioconductor.org/packages/release/bioc/html/gypsum.html)|[![Release OK](https://bioconductor.org/shields/build/release/bioc/gypsum.svg)](http://bioconductor.org/checkResults/release/bioc-LATEST/gypsum/)|
|[BioC-devel](https://bioconductor.org/packages/devel/bioc/html/gypsum.html)|[![Devel OK](https://bioconductor.org/shields/build/devel/bioc/gypsum.svg)](http://bioconductor.org/checkResults/devel/bioc-LATEST/gypsum/)|

Pretty much as it says on the can, provides an R client for the [**gypsum** REST API](https://github.com/ArtifactDB/gypsum-worker).
Readers are referred to the [API's documentation](https://gypsum-test.aaron-lun.workers.dev) or the [user guide](https://bioconductor.org/packages/devel/bioc/vignettes/gypsum/inst/doc/userguide.html) for more details.
